
import { NativeModules } from 'react-native';


var DEVICE_NOTIF_RECEIVED_EVENT = 'Pushbots-remoteNotificationReceived';
var DEVICE_NOTIF_OPENED_EVENT = 'Pushbots-remoteNotificationOpened';
var DEVICE_NOTIF_REG_EVENT = 'Pushbots-remoteNotificationsRegistered';
var RNPushbots = NativeModules.Pushbtos;
const _notifHandlers = new Map();

function handleConnectionStateChange(isConnected) {
    if (!isConnected) return;

    Pushbtos.configure();
    NetInfo.isConnected.removeEventListener('change', handleConnectionStateChange);
}

NetInfo.isConnected.fetch().then(isConnected => {
    if (isConnected) return Pushbtos.configure();
    NetInfo.isConnected.addEventListener('change', handleConnectionStateChange);
}).catch((...args) => console.warn("Error: ", args));


const { RNPushbotsReact } = NativeModules;

export default class Pushbtos {

    static addEventListener(type: any, handler: Function) {

        // Listen to events of notification received, opened, device registered and IDSAvailable.

        invariant(
            type === 'received' || type === 'opened' || type === 'registered' || type === 'ids',
            'Pushbtos only supports `received`, `opened`, `registered`, and `ids` events'
        );

        var listener;

        if (type === 'received') {
            listener = NativeAppEventEmitter.addListener(
                DEVICE_NOTIF_RECEIVED_EVENT,
                (notification) => {
                    handler(notification);
                }
            );
        } else if (type === 'opened') {
            listener = NativeAppEventEmitter.addListener(
                DEVICE_NOTIF_OPENED_EVENT,
                (result) => {
                    handler(result);
                }
            );
        } else if (type === 'registered') {
            listener = NativeAppEventEmitter.addListener(
                DEVICE_NOTIF_REG_EVENT,
                (notifData) => {
                    handler(notifData);
                }
            );
        } 
        _notifHandlers.set(type, listener);

    }

    static removeEventListener(type: any, handler: Function) {
        invariant(
            type === 'received' || type === 'opened' || type === 'registered' || type === 'ids',
            'Pushbtos only supports `received`, `opened`, `registered`, and `ids` events'
        );
        var listener = _notifHandlers.get(type);
        if (!listener) {
            return;
        }
        listener.remove();
        _notifHandlers.delete(type);
    }

    static registerForPushNotifications() {
        if (Platform.OS === 'ios') {
            RNPushbots.registerForRemoteNotifications();
        } else {
            console.log("This function is not supported on Android");
        }
    }

    static requestPermissions(permissions) {
        var requestedPermissions = {};
        if (Platform.OS === 'ios') {
            if (permissions) {
                requestedPermissions = {
                    alert: !!permissions.alert,
                    badge: !!permissions.badge,
                    sound: !!permissions.sound
                };
            } else {
                requestedPermissions = {
                    alert: true,
                    badge: true,
                    sound: true
                };
            }
            RNPushbots.requestPermissions(requestedPermissions);
        } else {
            console.log("This function is not supported on Android");
        }
    }

    static configure() {
        RNPushbots.configure();
    }

    static checkPermissions(callback: Function) {
        if (Platform.OS === 'ios') {
            invariant(
                typeof callback === 'function',
                'Must provide a valid callback'
            );
            RNPushbots.checkPermissions(callback);
        } else {
            console.log("This function is not supported on Android");
        }
    }

    static sendTag(key, value) {
        RNPushbots.sendTag(key, value);
    }

    static sendTags(tags) {
        RNPushbots.sendTags(tags || {});
    }

    static getTags(next) {
        RNPushbots.getTags(next);
    }

    static deleteTag(key) {
        RNPushbots.deleteTag(key);
    }

    static enableVibrate(enable) {
        if (Platform.OS === 'android') {
            RNPushbots.enableVibrate(enable);
        } else {
            console.log("This function is not supported on iOS");
        }
    }

    static enableSound(enable) {
        if (Platform.OS === 'android') {
            RNPushbots.enableSound(enable);
        } else {
            console.log("This function is not supported on iOS");
        }
    }


    static setSubscription(enable) {
        RNPushbots.setSubscription(enable);
    }

    static promptLocation() {
        //Supported in both iOS & Android
        RNPushbots.promptLocation();
    }

    //Android only: Set Display option of the notifications. displayOption is of type OSInFocusDisplayOption
    // 0 -> None, 1 -> InAppAlert, 2 -> Notification
    static inFocusDisplaying(displayOption) {
        if (Platform.OS === 'android') {
            RNPushbots.inFocusDisplaying(displayOption);
        }
    }

    static postNotification(contents, data, player_id, otherParameters) {
        if (Platform.OS === 'android') {
            RNPushbots.postNotification(JSON.stringify(contents), JSON.stringify(data), player_id, JSON.stringify(otherParameters));
        } else {
            RNPushbots.postNotification(contents, data, player_id, otherParameters);
        }
    }

    static clearOneSignalNotifications() {
        if (Platform.OS === 'android') {
            RNPushbots.clearOneSignalNotifications();
        } else {
            console.log("This function is not supported on iOS");
        }
    }

    // static cancelNotification(id) {
    //     if (Platform.OS === 'android') {
    //         RNPushbots.cancelNotification(id);
    //     } else {
    //         console.log("This function is not supported on iOS");
    //     }
    // }

    // //Sends MD5 and SHA1 hashes of the user's email address (https://documentation.onesignal.com/docs/ios-sdk-api#section-synchashedemail)
    // static syncHashedEmail(email) {
    //     RNPushbots.syncHashedEmail(email);
    // }

    // static setLogLevel(nsLogLevel, visualLogLevel) {
    //     RNPushbots.setLogLevel(nsLogLevel, visualLogLevel);
    // }


    

}